// Copyright (C) 2022 The Qt Company Ltd.
// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0

#pragma once

#define QDOC_CATCH_GENERATORS_ROOT_NAMESPACE qdoc::catch_generators

#define QDOC_CATCH_GENERATORS_PRIVATE_NAMESPACE details

#define QDOC_CATCH_GENERATORS_TRAITS_NAMESPACE traits

#define QDOC_CATCH_GENERATORS_QCHAR_ALPHABETS_NAMESPACE alphabets

#define QDOC_CATCH_GENERATORS_UTILITIES_ABSOLUTE_NAMESPACE QDOC_CATCH_GENERATORS_ROOT_NAMESPACE::QDOC_CATCH_GENERATORS_PRIVATE_NAMESPACE::utils
